"""
PnlCalculator class is used to calculate the PnL of a position.
"""


class Trade:
    """Clas to represent a trade."""

    def __init__(self, quantity, price):
        """initialise a trade with a quantity and price."""
        self.quantity = quantity
        self.price = price


class PnlCalculator:
    """Class to calculate the PnL of a position."""

    def __init__(self):
        """A PnlCalculator is initialised with no trades."""
        self.current_price = 0
        self.position_size = 0
        self.total_cost = 0
        self.realised_pnl_total = 0

    def add_trade(self, trade):
        """Add a trade to the PnlCalculator."""
        if (self.position_size * trade.quantity) < 0:  # Reducing position
            realised_quantity = min(abs(self.position_size), abs(trade.quantity))
            self.realised_pnl_total += (
                realised_quantity
                * (self.average_price - trade.price)
                * (-1 if trade.quantity < 0 else 1)
            )
            self.position_size += trade.quantity
            self.total_cost = self.average_price * self.position_size
        else:  # Increasing position
            self.position_size += trade.quantity
            self.total_cost += trade.quantity * trade.price
        # If the position size is 0 (i.e., all trades are closed), reset the total cost
        if self.position_size == 0:
            self.total_cost = 0

    def update_price(self, price):
        """Update the current price of the PnlCalculator."""
        self.current_price = price

    @property
    def average_price(self):
        """Calculate the average price of the open position."""
        if self.position_size == 0:
            return 0
        return self.total_cost / self.position_size

    @property
    def unrealised_pnl(self):
        """Calculate the unrealised PnL of the PnlCalculator."""
        return self.position_size * (self.current_price - self.average_price)

    @property
    def realised_pnl(self):
        """Return the realised PnL of the PnlCalculator."""
        return self.realised_pnl_total



"""
Module to interact with the Rysk subgraph.
"""
import json
from dataclasses import dataclass
from typing import Any, Dict, List

import requests

from rysk_client.src.constants import DEFAULT_TIMEOUT, SUBGRAPH_URL

MARKET_SUBGRAPH_QUERY = """
{series 
  {   
    id 
    expiration 
    netDHVExposure 
    strike
    isPut
    isBuyable
    isSellable
  }
}
"""

SHORT_SUBGRAPH_QUERY = """
{
  shortPositions(
    first: 1000,
    where: {
         account: "%s",
         oToken_: {expiryTimestamp_gte: "1683273600"}
         }
    ){
        id
        netAmount
        buyAmount
        sellAmount
        active
        realizedPnl
        oToken {
            id
            symbol
            expiryTimestamp
            strikePrice
            isPut
            underlyingAsset {
                id
            }
            createdAt
        }
        settleActions {
            id
        }
        optionsBoughtTransactions {
            amount
            premium
        }
        optionsSoldTransactions {
            amount
            premium
        }
  }
}
"""

LONG_SUBGRAPH_QUERY = """
{
  longPositions(
    first: 1000,
    where: {
         account: "%s",
         oToken_: {expiryTimestamp_gte: "1683273600"}
         }
    ){
        id
        netAmount
        buyAmount
        sellAmount
        active
        realizedPnl
        oToken {
            id
            symbol
            expiryTimestamp
            strikePrice
            isPut
            underlyingAsset {
                id
            }
            createdAt
        }
        redeemActions {
            id
        }
        optionsBoughtTransactions {
            amount
            premium
        }
        optionsSoldTransactions {
            amount
            premium
        }
  }
}
"""


@dataclass
class SubgraphClient:
    """Simple client to interact with the Rysk subgraph."""

    url: str = SUBGRAPH_URL

    def _query(self, query):
        """Simple function to call a subgraph query."""
        headers = {"Content-Type": "application/json"}
        subgraph_query = {"query": query}
        response = requests.post(
            url=self.url,
            headers=headers,
            data=json.dumps(subgraph_query),
            timeout=DEFAULT_TIMEOUT,
        )
        data = json.loads(response.content)["data"]
        return data

    def query_markets(self):
        """Query the subgraph for markets."""
        return self._query(MARKET_SUBGRAPH_QUERY)["series"]

    def query_longs(self, address: str) -> List[Dict[str, Any]]:
        """Query the subgraph for longs."""
        query = LONG_SUBGRAPH_QUERY % address.lower()
        result = self._query(query)
        return result["longPositions"]

    def query_shorts(self, address: str) -> List[Dict[str, Any]]:
        """Query the subgraph for shorts."""
        query = SHORT_SUBGRAPH_QUERY % address.lower()
        result = self._query(query)
        return result["shortPositions"]
